import BaseController from 'common/base.controller'
import { role, router } from 'decorators/router'
import { BlockChain } from 'chain/BlockChain'
import { ZError } from 'common/ZError'
import { RedisClient } from 'redis/RedisClient'

class ChainController extends BaseController {
  /**
   * 活动用的接口, 检查用户是否有已claim NFT
   * 如果有，返回tokenId
   * 否则, 返回0
   */
  @role('anon')
  @router('post /chain/query_info')
  @router('get /chain/query_info')
  async queryUserInfo(req, res) {
    let { address, contract } = req.params
    if (!address) {
      throw new ZError(10, 'address is required')
    }
    let key = `claim_${address.toLowerCase()}`
    let tokenStr = await new RedisClient().get(key)
    if (tokenStr) {
      return { id: parseInt(tokenStr) }
    }
    let info = await new BlockChain().activityReactor.getMintableCount({ user: address, address: contract })
    if (parseInt(info) > 0) {
      await new RedisClient().set(key, info)
    }
    return { id: parseInt(info) }
  }
}
export default ChainController
