import { getModelForClass, DocumentType, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

@dbconn()
@index({ address: 1, chain: 1 }, { unique: false })
@index({ chain: 1, address: 1, user: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'ft_holder', timestamps: true },
})
export class FtHolderClass extends BaseModule {
  @prop()
  public chain: number
  @prop({ required: true })
  public address!: string
  @prop()
  public user: string
  @prop({ default: 18 })
  public decimals: number
  @prop({ default: '0' })
  public amount: string
  @prop({ default: 0 })
  public version: number

  public async incAmount(this: DocumentType<FtHolderClass>, val: string) {
    this.amount = (BigInt(this.amount) + BigInt(val)).toString()
  }

  public async decAmount(this: DocumentType<FtHolderClass>, val: string) {
    this.amount = (BigInt(this.amount) - BigInt(val)).toString()
  }
}

export const FtHolder = getModelForClass(FtHolderClass, {
  existingConnection: FtHolderClass['db'],
})
