import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'
import { NftHolder } from './NftHolder'

@dbconn()
@index({ address: 1, chain: 1 }, { unique: false })
@index({ chain: 1, transactionHash: 1, address: 1, tokenId: 1, from: 1, to: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'nft_transfer_event', timestamps: true },
})
export class NftTransferEventClass extends BaseModule {
  @prop({ required: true })
  public chain: number
  @prop({ required: true })
  public address!: string
  @prop()
  public event: string
  @prop({ required: true })
  public transactionHash: string
  @prop()
  public transactionIndex: number
  @prop()
  public logIndex: number
  @prop()
  public blockNumber: number
  @prop()
  public blockHash: string
  @prop()
  public removed: boolean
  @prop()
  public from: string
  @prop()
  public to: string
  @prop()
  public tokenId: string
  @prop()
  public eventId: string
  @prop()
  public blockTime: number
  @prop({ default: 0 })
  public version: number

  public static async saveEvent(event: any) {
    const tokenId = event.tokenId || event.returnValues?.tokenId
    if (!tokenId) {
      return
    }
    const from = (event.source || event.returnValues?.from).toLowerCase()
    const to = (event.target || event.returnValues?.to).toLowerCase()
    const address = (event.address || event.tokenAddress).toLowerCase()
    const data = {
      chain: event.chain,
      address,
      blockNumber: event.blockHeight || event.blockNumber,
      blockHash: event.blockHash,
      removed: event.removed,
      from,
      to,
      transactionHash: event.transactionHash || event.hash,
      transactionIndex: event.transactionIndex,
      tokenId,
      eventId: event.id,
      logIndex: event.logIndex,
      blockTime: event.timestamp,
      $inc: { version: 1 },
    }

    await NftTransferEvent.insertOrUpdate(
      {
        chain: event.chain,
        transactionHash: event.transactionHash || event.hash,
        address,
        tokenId,
        from,
        to,
      },
      data,
    )
    if (from !== to) {
      await NftHolder.insertOrUpdate(
        { chain: event.chain, address, tokenId },
        {
          user: to,
          lastTxHash: event.transactionHash || event.hash,
          $inc: { version: 1 },
        },
      )
    }
    return true
  }
}

export const NftTransferEvent = getModelForClass(NftTransferEventClass, {
  existingConnection: NftTransferEventClass['db'],
})
