import { NftHolder } from 'models/NftHolder'
import { RequestTask } from 'models/RequestTask'

const graphql = require('graphql')
import { getFirstToken } from 'service/chain.service'

const { GraphQLSchema, GraphQLObjectType, GraphQLString, GraphQLList } = graphql

const nftType = new GraphQLObjectType({
  name: 'erc721',
  fields: () => ({
    address: { type: GraphQLString },
    contract: { type: GraphQLString },
    id: { type: GraphQLString },
    chain: { type: GraphQLString },
  }),
})

// Define Root Query
const RootQuery = new GraphQLObjectType({
  name: 'RootQueryType',
  fields: {
    nft: {
      type: nftType,
      args: { address: { type: GraphQLString }, contract: { type: GraphQLString } },
      async resolve(parent, args) {
        return await getFirstToken(args.address, args.contract)
      },
    },
    nfts: {
      type: new GraphQLList(nftType),
      args: {
        address: { type: GraphQLString },
        contract: { type: GraphQLString },
        id: { type: GraphQLString },
        chain: { type: GraphQLString },
      },
      async resolve(parent, args) {
        return await NftHolder.queryNFT(args)
      },
    },
  },
})

// const Mutations = new GraphQLObjectType({
//   name: 'Mutations',
//   fields: {
//     addCar: {
//       type: nftType,
//       args: {},
//       async resolve(parent, args) {
//         const data = await RequestTask.addCar(args)
//         return data
//       },
//     },
//   },
// })

module.exports = new GraphQLSchema({
  query: RootQuery,
})
