import { BlockChain } from 'chain/BlockChain'
import logger from 'logger/logger'
import { ChainTask } from 'models/ChainTask'
import { RequestTask } from 'models/RequestTask'
import { ChainQueue } from 'queue/chain.queue'

export async function restartAllUnFinishedTask() {
  let chainTasks = await ChainTask.allUnFinishedTask()
  logger.info(`restore ${chainTasks.length} chain tasks`)
  let chainQueue = new ChainQueue()
  for (let cTask of chainTasks) {
    let subTasks = await RequestTask.allUnFinishedTask(cTask.id)
    logger.info(`restore ${subTasks.length} req tasks fro ${cTask.id}`)
    for (let subTask of subTasks) {
      chainQueue.addTaskToQueue(subTask)
    }
  }
}

export async function getFirstToken(address: string, contract?: string) {
  let tokenId = ''
  contract = contract || process.env.CHAIN_NFT_ADDRESS
  try {
    tokenId = await new BlockChain().erc721Reactor.getCollectibleTokenId({
      address: contract,
      selectedAddress: address,
      index: 0,
    })
  } catch (err) {
    logger.info(err.message || err)
  }

  return {
    address,
    contract,
    id: tokenId,
  }
}
