import assert from 'assert'
import { AllChains } from 'chain/allchain'
import { HttpRetryProvider } from 'chain/HttpRetryProvider'
import logger from 'logger/logger'
import { RedisClient } from 'redis/RedisClient'

import { clearTimeCache, getPastEventsIter, processEvents } from 'utils/contract.util'
import Web3 from 'web3'

export class EventSyncSvr {
  web3: Web3
  provider: HttpRetryProvider
  fromBlock: number = 27599018
  toBlock: number = 10000
  contract: any
  event: string
  blockKey = ''
  address: string
  chain: number
  eventProcesser: any

  constructor({
    chain,
    address,
    event,
    abi,
    fromBlock,
    eventProcesser,
  }: {
    chain: number
    address: string
    event: string
    abi: any
    fromBlock: number
    eventProcesser: any
  }) {
    this.chain = chain
    const chainData = AllChains.find(o => o.id === chain)
    assert(chainData, `chain data with ${this.chain} not found`)
    this.provider = new HttpRetryProvider(chainData.rpc.split('|'))
    // @ts-ignore
    this.web3 = new Web3(this.provider)
    this.contract = new this.web3.eth.Contract(abi, address)
    this.address = this.contract.options.address
    this.event = event
    this.fromBlock = fromBlock
    this.blockKey = `${chain}_${address.toLowerCase()}_${event}`
    this.eventProcesser = eventProcesser
  }

  async parseEvents() {
    let currentBlock = await this.web3.eth.getBlockNumber()
    let blockStr = await new RedisClient().get(this.blockKey)
    if (blockStr) {
      this.fromBlock = Math.max(parseInt(blockStr), this.fromBlock)
    }
    this.toBlock = currentBlock
    if (this.fromBlock > this.toBlock) {
      return
    }
    logger.log(`query events:: ${this.event} address: ${this.address} from: ${this.fromBlock} to: ${this.toBlock}`)
    let events = getPastEventsIter({
      chain: this.chain,
      contract: this.contract,
      event: this.event,
      fromBlock: this.fromBlock,
      toBlock: this.toBlock,
    })
    // this.fromBlock = this.toBlock
    await processEvents(this.web3, events, this.chain, this.eventProcesser.saveEvent)
    // 处理完一种nft后, 清楚block的timestamp缓存
    clearTimeCache()
  }
}
