import { singleton } from 'decorators/singleton'
import logger from 'logger/logger'
import { createTransport, Transporter } from 'nodemailer'
import Mail from 'nodemailer/lib/mailer'

@singleton
export class MailService {
  private transporter: Transporter
  constructor() {
    const options = {
      host: process.env.MAIL_SMTP_HOST,
      secure: true,
      auth: {
        user: process.env.MAIL_SMTP_USER,
        pass: process.env.MAIL_SMTP_PASS,
      },
      logger: true,
      debug: false,
    }
    // @ts-ignore
    this.transporter = createTransport(options, {})
  }

  public async send(message: Mail.Options) {
    logger.info('begin send mail: ')
    logger.info(JSON.stringify(message))
    await this.transporter.verify()
    return this.transporter.sendMail(message)
  }
}
